#include "../include/error.h"
#include "../include/util.h"
#include "../include/pool.h"
#include <stdio.h>
#include <stdlib.h>

void lm_pool_free(lm_pool_t *pool){
  lm_url_free(&pool->url);
  free(pool);
}

lm_pool_t *lm_pool_new(char *name, char *url){
  lm_pool_t *pool = malloc(sizeof(lm_pool_t));
  if(!lm_url_parse(&pool->url, url)){
    free(pool);
    return NULL;
  }

  if(!eq(pool->url.protocol, "mptp")){
    lm_error_set(LM_ERR_PoolNoSupport);
    lm_pool_free(pool);
    return NULL;
  }

  pool->name = name;
  return pool;
}

bool lm_ctx_pool_add(lm_ctx_t *ctx, char *name, char *url){
  lm_pool_t *pool = lm_pool_new(name, url);
  if(NULL == pool)
    return false;

  pdebug(ctx, __func__, "pool name is %s", pool->name);
  pdebug(ctx, __func__, "pool URL is %s://%s%s", pool->url.protocol, pool->url.host, pool->url.path);

  if(NULL == ctx->pools){
    ctx->pools = pool;
    return true;
  }

  lm_pool_t *cur = ctx->pools;
  while(NULL != cur) {
    if(NULL == cur->next){
      cur->next = pool;
      return true;
    }
    cur = cur->next;
  }

  return false;
}

bool lm_ctx_pool_del(lm_ctx_t *ctx, char *name){
  lm_pool_t *cur = ctx->pools, *prev = NULL;
  while(NULL != cur) {
    if(!eq(cur->name, name)){
      cur = cur->next;
      continue;
    }

    if(NULL == prev){
      ctx->pools = cur->next;
    lm_pool_free(cur);
      return true;
    }

    prev->next = cur->next;
    lm_pool_free(cur);
    return true;
  }

  return false;
}

void lm_ctx_pool_clear(lm_ctx_t *ctx){
  lm_pool_t *cur = ctx->pools, *prev = NULL;
  while(NULL != cur) {
    prev = cur;
    cur = cur->next;
    lm_pool_free(prev);
  }

  ctx->pools = NULL;
}

void lm_ctx_pool_test(lm_ctx_t *ctx){
  return;
}
